# reads in a list of words, binary search for user-supplied word
# assumes sorted list
# also print number of words checked for each search
# and at the end, the average number of words checked per search
# ECS 10, May 22, 2009
# Matt Bishop
import string

# this is global just to emphasize how that works
wordlist = []

# this counts how many words you check before finding the word
# (or giving up)
wordcount = 0

# search a list of words for a word (binary search)
# parameters: word, the word to find in the list
# returns: index (+1) of word in list
#          -1 if not in list
# globals: wordlist, list to search
def search(word):
        # declarations
        global wordlist, wordcount
        # get the endpoints (low, high indices) of the list
        low = 0
        high = len(wordlist) - 1
        # if low ever passes high, the word isn't in the list
        while low <= high:
                # one more word looked at
                wordcount += 1
                # check the middle word
                mid = (low + high) / 2
                if word == wordlist[mid]:
                        # match -- we're done!
                        return mid + 1
                elif word < wordlist[mid]:
                        # word comes before middle word
                        # reduce the high index of the interval
                        high = mid - 1
                else:
                        # word comes after middle word
                        # advance the low index of the interval
                        low = mid + 1
        # didn't find squat -- report failure abjectly
        return -1
                

# append words from a file to a list; assumes one word per line
# note: lines separated by *newlines*, not *carriage returns*
# parameters: none
# returns: True if words loaded successfully
#          False if problem opening file
# globals: wordlist, list to append words to
def loadfile():
        # declaration
        global wordlist
        # open the file, trapping errors
        try:
                infile = open("list.txt", "r")
        except IOError, errmsg:
                # say why it failed, return error code
                print "Could not open 'xxx.txt':", errmsg
                return False
        # go through the file line by line
        for i in infile:
                # strip any trailing white space (inc. newline)
                i = string.rstrip(i)
                # tack the word onto the list
                wordlist.append(i)
        # all done! close file and return success code
        infile.close()
        return True


# this puts it all together
def main():
        # declaration
        global wordlist, wordcount
        # load the word list; on error, quit
        if not loadfile():
                return
        # this counts total words checked across all searches
        totalwords = 0
        # count number of searches to compute average number
        # of words examined per search
        nsearch = 0
        # loop 
        while True:
                # read in word; quit on EOF
                try:
                        find = raw_input("What word should I look for EOF to quit)? ")
                except EOFError:
                        break
                # count number of words searched for
                nsearch += 1
                # initialize number of words checked
                wordcount = 0
                # find the word if it's there
                found = search(find)
                # announce result
                if found == -1:
                        print "'%s' is not in the list" % find
                else:
                        print "'%s' is word number %d" % (find, found)
                # print number of words checked
                if wordcount == 1:
                        print "Checked 1 word"
                else:
                        print "Checked", wordcount, "words"
                # increment total words checked
                totalwords += wordcount
        # give the average number of words checked per search
        print "On average, checked", float(totalwords)/nsearch, "words"

main()
